package eu.jonahbauer.wizard.client.libgdx.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.utils.viewport.ExtendViewport;
import com.badlogic.gdx.utils.viewport.FitViewport;
import eu.jonahbauer.wizard.client.libgdx.MenuAtlas;
import eu.jonahbauer.wizard.client.libgdx.UiskinAtlas;
import eu.jonahbauer.wizard.client.libgdx.WizardGame;

public abstract class MenuScreen implements Screen {
    protected final float BUTTON_BAR_Y = WizardGame.HEIGHT * 0.15f;

    protected WizardGame game;

    // shared data between all menu screens
    protected static class Data {
        public final ExtendViewport extendViewport;
        public final FitViewport fitViewport;

        public final Sound buttonClickSound;

        public final TextureAtlas uiskinAtlas;
        public final TextureAtlas menuAtlas;

        public final TextureRegion title;
        public final TextureRegion background;
        public final TextureRegion[] corners = new TextureRegion[4];
        public final TextureRegion[] symbols = new TextureRegion[4];

        public final Skin skin;
        public final Stage stage;

        public Data() {
            this.extendViewport = new ExtendViewport(WizardGame.WIDTH, WizardGame.HEIGHT);
            this.fitViewport = new FitViewport(WizardGame.WIDTH, WizardGame.HEIGHT);

            this.buttonClickSound = Gdx.audio.newSound(Gdx.files.internal("button_click_s.mp3"));

            this.uiskinAtlas = new TextureAtlas(Gdx.files.internal(UiskinAtlas.$PATH));
            this.menuAtlas = new TextureAtlas(Gdx.files.internal(MenuAtlas.$PATH));

            this.title = menuAtlas.findRegion(MenuAtlas.TITLE);
            this.background = menuAtlas.findRegion(MenuAtlas.BACKGROUND);
            this.corners[0] = menuAtlas.findRegion(MenuAtlas.ECKE_LO);
            this.corners[1] = menuAtlas.findRegion(MenuAtlas.ECKE_LU);
            this.corners[2] = menuAtlas.findRegion(MenuAtlas.ECKE_RU);
            this.corners[3] = menuAtlas.findRegion(MenuAtlas.ECKE_RO);
            this.symbols[0] = menuAtlas.findRegion(MenuAtlas.SYMBOL0);
            this.symbols[1] = menuAtlas.findRegion(MenuAtlas.SYMBOL1);
            this.symbols[2] = menuAtlas.findRegion(MenuAtlas.SYMBOL2);
            this.symbols[3] = menuAtlas.findRegion(MenuAtlas.SYMBOL3);

            this.skin = new Skin(Gdx.files.internal("uiskin.json"), uiskinAtlas);
            this.skin.getAll(BitmapFont.class).forEach(entry ->
                entry.value.getRegion().getTexture().setFilter(Texture.TextureFilter.Linear, Texture.TextureFilter.Linear)
            );
            this.stage = new Stage(fitViewport);
        }

        public void reset() {
            stage.clear();
        }

        public void dispose() {
            buttonClickSound.dispose();
            uiskinAtlas.dispose();
            menuAtlas.dispose();
            skin.dispose();
            stage.dispose();
        }
    }
    protected Data data;
    protected boolean dispose;

    public MenuScreen(WizardGame game) {
        this.game = game;
    }

    protected MenuScreen(WizardGame game, Data data) {
        this.game = game;
        this.data = data;
    }

    @Override
    public void show() {
        if (data == null) {
            data = new Data();
        } else {
            data.reset();
        }
        if (WizardGame.DEBUG) {
            data.stage.setDebugAll(true);
            data.stage.addListener(event -> {
                if (!(event instanceof InputEvent input) || input.getType() == InputEvent.Type.mouseMoved) return false;

                var actor = event.getTarget();
                if (actor != null) {
                    System.out.printf(
                            "%s(name=%s, x=%f, y=%f, width=%f, height=%f)%n",
                            actor.getClass().getSimpleName(),
                            actor.getName(),
                            actor.getX(),
                            actor.getY(),
                            actor.getWidth(),
                            actor.getHeight()
                    );
                }
                return false;
            });
        }
    }

    @Override
    public final void render(float delta) {
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);

        data.extendViewport.apply(true);
        game.batch.setProjectionMatrix(data.extendViewport.getCamera().combined);
        game.batch.begin();
        float scale = Math.max(
                data.extendViewport.getWorldWidth() / WizardGame.WIDTH,
                data.extendViewport.getWorldHeight() / WizardGame.HEIGHT
        );
        game.batch.draw(data.background, 0, 0, scale * WizardGame.WIDTH, scale * WizardGame.HEIGHT);
        game.batch.draw(data.corners[0], 0, data.extendViewport.getWorldHeight() - data.corners[0].getRegionHeight());
        game.batch.draw(data.corners[1], 0, 0);
        game.batch.draw(data.corners[2], data.extendViewport.getWorldWidth() - data.corners[2].getRegionWidth(), 0);
        game.batch.draw(data.corners[3], data.extendViewport.getWorldWidth() - data.corners[3].getRegionWidth(), data.extendViewport.getWorldHeight() - data.corners[3].getRegionHeight());
        game.batch.end();

        data.fitViewport.apply();
        game.batch.setProjectionMatrix(data.fitViewport.getCamera().combined);
        game.batch.begin();
        game.batch.draw(data.title, 555, WizardGame.HEIGHT - 192 - 96, 810, 192);
        renderInternal(delta);
        game.batch.end();

        data.stage.act(delta);
        data.stage.draw();
    }

    protected abstract void renderInternal(float delta);

    @Override
    public final void resize(int width, int height) {
        data.extendViewport.update(width, height);
        data.fitViewport.update(width, height);
    }

    @Override
    public void pause() {

    }

    @Override
    public void resume() {

    }

    @Override
    public void hide() {

    }

    @Override
    public void dispose() {
        if (dispose) {
            data.dispose();
        }
    }

    protected void sfxClick() {
        data.buttonClickSound.play(0.6f);
    }
}
