package eu.jonahbauer.wizard.server;

import eu.jonahbauer.wizard.common.messages.data.PlayerData;
import eu.jonahbauer.wizard.common.messages.data.SessionData;
import eu.jonahbauer.wizard.common.messages.server.*;
import eu.jonahbauer.wizard.common.model.Configuration;
import eu.jonahbauer.wizard.server.machine.Player;
import lombok.AccessLevel;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.Getter;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ThreadLocalRandom;

@Getter
@EqualsAndHashCode(of = "uuid")
public class Session {
    private static final int MIN_PLAYERS = 3;
    private static final int MAX_PLAYERS = 6;

    private final UUID uuid;
    private final String name;
    private final long timeout;
    private final Configuration configuration;

    @Getter(AccessLevel.NONE)
    private final Map<UUID, SessionPlayer> players = new HashMap<>();


    public Session(UUID uuid, String name, long timeout, Configuration configuration) {
        this.uuid = uuid;
        this.name = name;
        this.timeout = timeout;
        this.configuration = configuration;
    }

    /**
     * Associates the given player with this session, removes him from the lobby, notifies all other players in the
     * session with a {@link PlayerJoinedMessage}, the joining player with a {@link SessionJoinedMessage} and all
     * players in the lobby with a {@link SessionModifiedMessage}.
     *
     * @param player the player
     * @param name the players chosen name
     * @return the players uuid
     */
    public synchronized UUID join(Player player, String name) {
        if (players.size() == MAX_PLAYERS) {
            throw new NackException(NackMessage.BAD_REQUEST, "Session is full.");
        } else if (players.values().stream().anyMatch(p -> p.getName().equalsIgnoreCase(name))) {
            throw new NackException(NackMessage.BAD_REQUEST, "Name is already taken.");
        }

        Lobby.getInstance().leave(player);

        SessionPlayer sessionPlayer;
        do {
            sessionPlayer = new SessionPlayer(UUID.randomUUID(), name, player);
        } while (players.putIfAbsent(sessionPlayer.getUuid(), sessionPlayer) != null);

        notifyJoined(sessionPlayer.toData());
        Lobby.getInstance().notifyPlayers(new SessionModifiedMessage(toData()));

        return sessionPlayer.getUuid();
    }

    public synchronized void leave(UUID player) {
        if (players.remove(player) != null) {
            if (players.size() == 0) {
                Lobby.getInstance().removeSession(uuid);
            } else {
                notifyPlayers(new PlayerLeftMessage(player));
                Lobby.getInstance().notifyPlayers(new SessionModifiedMessage(toData()));
            }
        }
    }

    public synchronized void ready(UUID player, boolean ready) {
        var sessionPlayer = players.get(player);
        if (sessionPlayer == null) {
            throw new NackException(NackMessage.BAD_REQUEST, "Who are you?");
        }

        if (sessionPlayer.isReady() != ready) {
            sessionPlayer.setReady(ready);
            notifyPlayers(new PlayerModifiedMessage(sessionPlayer.toData()));
        }

        if (players.size() >= MIN_PLAYERS && players.values().stream().allMatch(SessionPlayer::isReady)) {
            // TODO start game
        }
    }

    private void notifyJoined(PlayerData joined) {
        var message = new PlayerJoinedMessage(joined);
        for (SessionPlayer player : players.values()) {
            if (player.getUuid().equals(joined.getUuid())) {
                player.getPlayer().send(new SessionJoinedMessage(
                        getUuid(),
                        player.getUuid(),
                        players.values().stream().map(SessionPlayer::toData).toList(),
                        player.getSecret()
                ));
            } else {
                player.getPlayer().send(message);
            }
        }
    }

    private void notifyPlayers(ServerMessage message) {
        for (SessionPlayer player : players.values()) {
            player.getPlayer().send(message);
        }
    }

    public SessionData toData() {
        return new SessionData(uuid, name, players.size(), configuration);
    }

    @Data
    @EqualsAndHashCode(of = "uuid")
    private static class SessionPlayer {
        private final UUID uuid;
        private final String name;
        private final Player player;
        private final String secret = generateSecret();
        private boolean ready;

        private static String generateSecret() {
            return ThreadLocalRandom.current()
                    .ints(32, 'a', 'z' + 1)
                    .collect(StringBuilder::new, (sb, i) -> sb.append((char) i), StringBuilder::append)
                    .toString();
        }

        public PlayerData toData() {
            return new PlayerData(uuid, name, ready);
        }
    }
}
